# Copyright (c) 2008-2009 gocept gmbh & co. kg
# See also LICENSE.txt

from gocept.directory.app import locate_directory_from
import gocept.directory.form
import gocept.directory.interfaces
import grok
import megrok.pagelet
import zope.component
import zope.interface


class ResourceGroup(grok.Container):

    grok.implements(gocept.directory.interfaces.IResourceGroup)

    def __init__(self, name=''):
        super(ResourceGroup, self).__init__()
        self.name = name

    @property
    def nodes(self):
        for obj in self.values():
            if gocept.directory.interfaces.INode.providedBy(obj):
                yield obj

    def suggest_hostname(self):
        if not self.name:
            return ''
        nodenames = set(n.name for n in self.nodes) | set(self.keys())
        index = 0
        name = '%s%02d' % (self.name, index)
        while name in nodenames:
            index += 1
            name = '%s%02d' % (self.name, index)
        return name

    def register_utility(self, site_manager):
        site_manager.registerUtility(self,
                gocept.directory.interfaces.IResourceGroup, self.name)


@grok.subscribe(gocept.directory.interfaces.IResourceGroup,
        grok.IObjectAddedEvent)
def resourcegroup_added(rg, event):
    directory = locate_directory_from(rg)
    rg.register_utility(directory.getSiteManager())


class AddResourceGroup(gocept.directory.form.AddForm):

    grok.context(gocept.directory.interfaces.IDirectory)

    form_fields = grok.AutoFields(gocept.directory.interfaces.IResourceGroup)
    factory = ResourceGroup

    def chooseName(self, obj):
        return 'rg-' + obj.name


class Index(megrok.pagelet.Pagelet):

    grok.require('gocept.directory.ManageDirectory')


class DeleteNode(grok.View):
    grok.require('gocept.directory.ManageDirectory')
    grok.context(ResourceGroup)
    grok.name('delete_node')

    def __call__(self):
        if self.request.method != 'POST':
            raise ValueError('delete_node must be called via POST')
        nodeid = self.request.form['node']
        nodetitle = self.context[nodeid].title
        del self.context[nodeid]
        self.flash('Node "%s" has been deleted.' % nodetitle)
        self.request.response.redirect(self.url(self.context))

    def render(self):
        pass


@grok.subscribe(gocept.directory.interfaces.IDirectory, grok.IObjectAddedEvent)
def install_default_resource_group(directory, event):
    directory['rg-services'] = ResourceGroup('services')
